/*boundary_interpolate - run boundary_blend on a whole directory.
  Copyright (c) 1993 Matthew Belmonte

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

/*Note that the interpolation factor is a FACTOR, i.e., an interpolation factor
  of 1 will produce zero interpolated slices, and in general an interpolation
  factor of N will produce N-1 interpolated slices.*/
#define MAX_INT_FACTOR 4	/*Feel free to increase this, but if you do,
				  then alter the execl() call in main()
				  appropriately.*/

static unsigned NUM_SLICES=56, INT_FACTOR=4; /*default values*/
static char blend[] = "/usr/local/bin/boundary_blend";

#include <sys/param.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <ulocks.h>
#include <malloc.h>
#define is_digit(d) ((d >= '0') && (d <= '9'))

typedef int Boolean;
#define true 1
#define false 0

/*Process the option string that begins with argv[arg_num], and return false
  iff the option string is well-formed.*/
Boolean check_option(arg_num, argv)
int arg_num;
char *argv[];
  {
  if(argv[arg_num][0] != '-') return(true);
  switch(argv[arg_num][1])
    {
    case 'n':
      NUM_SLICES = atoi(argv[arg_num+1]);
      return(false);
    case 'i':
      INT_FACTOR = atoi(argv[arg_num+1]);
      return(false);
    default: return(true);
    }
  }

/*Horrid awful boring stuff to handle all possible options in setting
  parameters.  First check the argument count to make sure that the user has
  specified an image directory.  Once that's settled, go to that directory.
  Get the number of slices and interpolation factor from the _dimensions
  file, if it exists.  If not, use the default values 56 and 4, respectively.
  Process any option flags (this may involve overwriting the pre-set values for
  number of slices and interpolation factor), and then update the _dimensions
  file to reflect the new number of slices and zero on-line interpolations.
*/
void set_parameters(argc, argv)
int argc;
char *argv[];
  {
  FILE *dimensions;
  struct stat stbuf;
  Boolean use_dimensions_file;
  char *dimensions_buf, *dim_ptr;
  unsigned xsize, ysize, bits_per_voxel;
  char dimensions_file[MAXPATHLEN], new_dimensions_file[MAXPATHLEN];
  if(argc < 3)
    {
    printf("usage: %s [-n <number of slices>] [-i <interpolation factor>] <image directory> <head directory>\n", argv[0]);
    exit(1);
    }
  use_dimensions_file = false;
  dimensions_buf = (char *)0;
  sprintf(dimensions_file, "%s/_dimensions", argv[argc-2]);
  sprintf(new_dimensions_file, "%s/_dimensions~", argv[argc-2]);
  if((stat(dimensions_file, &stbuf) == 0)
	&& ((dimensions_buf = (char *)malloc(stbuf.st_size+1)) != (char *)0)
	&& ((dimensions = fopen(dimensions_file, "r")) != NULL))
    {
    if(fread(dimensions_buf, 1, stbuf.st_size, dimensions) == stbuf.st_size)
      {
      use_dimensions_file = true;
      for(dim_ptr = dimensions_buf; !is_digit(*dim_ptr); dim_ptr++);
      sscanf(dim_ptr, "%u %u %u %u %u", &NUM_SLICES, &xsize, &ysize, &INT_FACTOR, &bits_per_voxel);
      INT_FACTOR++;
      *(dim_ptr++) = '\0';
      while(*dim_ptr != '\n')
	dim_ptr++;
      dimensions_buf[stbuf.st_size] = '\0';
      }
    fclose(dimensions);
    }
  if((argc == 4) || (argc == 6) || (argc > 7)
	|| ((argc == 5) && check_option(1, argv))
	|| ((argc == 7) && (check_option(1, argv) || check_option(3, argv))))
    {
    printf("usage: %s [-n <number of slices>] [-i <interpolation factor>] <image directory> <head directory>\n", argv[0]);
    exit(1);
    }
  if(INT_FACTOR > MAX_INT_FACTOR)
    {
    fprintf(stderr, "%s: Increase MAX_INT_FACTOR to %d and recompile.\n", argv[0], INT_FACTOR);
    exit(1);
    }
  if(use_dimensions_file && (rename(dimensions_file, new_dimensions_file) == 0) && ((dimensions = fopen(dimensions_file, "w")) != NULL))
    {
    fprintf(dimensions, "%s%u\t\t%u\t\t%u\t\t0\t\t%u%s",
	dimensions_buf, INT_FACTOR*(NUM_SLICES-1)+1, xsize, ysize, bits_per_voxel, dim_ptr);
    fclose(dimensions);
    }
  if(dimensions_buf != (char *)0)
    free(dimensions_buf);
  }

unsigned new_slice_number(source_slice)
unsigned source_slice;
  {
  return(INT_FACTOR*(source_slice-1)+1);
  }

void main(argc, argv)
int argc;
char *argv[];
  {
  register unsigned i, slice;
  char source_file_name[MAXPATHLEN], source_file_name2[MAXPATHLEN], head_file_name[MAXPATHLEN], head_file_name2[MAXPATHLEN], dest_file_name[MAX_INT_FACTOR-1][MAXPATHLEN];
  set_parameters(argc, argv);
  for(slice = NUM_SLICES; slice != 1; slice--)
    {
    sprintf(source_file_name, "%s/%u.bw", argv[argc-2], slice);
    sprintf(dest_file_name[0], "%s/%u.bw", argv[argc-2], new_slice_number(slice));
    if(rename(source_file_name, dest_file_name[0]) == -1)
      {
      fprintf(stderr, "%s: couldn't find slice number %u\n", argv[0], slice);
      perror((char *)0);
      exit(1);
      }
    }
  for(slice = 1; slice != NUM_SLICES; slice++)
    {
    sprintf(source_file_name, "%s/%u.bw", argv[argc-2], new_slice_number(slice));
    sprintf(source_file_name2, "%s/%u.bw", argv[argc-2], new_slice_number(slice+1));
    sprintf(head_file_name, "%s/%u.bw", argv[argc-1], slice);
    sprintf(head_file_name2, "%s/%u.bw", argv[argc-1], slice+1);
    for(i = 1; i != INT_FACTOR; i++)
      sprintf(dest_file_name[i-1], "%s/%u.bw", argv[argc-2], new_slice_number(slice) + i);
    if(i != MAX_INT_FACTOR)
      dest_file_name[i-1][0] = '\0';
    switch(fork())
      {
      case 0: execl(blend, blend, source_file_name, source_file_name2, head_file_name, head_file_name2,
	dest_file_name[0][0] ? dest_file_name[0] : (char *)0,
	dest_file_name[1][0] ? dest_file_name[1] : (char *)0,
	dest_file_name[2][0] ? dest_file_name[2] : (char *)0,
	(char *)0);
      case -1: perror((char *)0); exit(1);
      }
    wait((int *)0);
    }
  exit(0);
  }
