/*gecon.c - convert a series of G.E. Signa MR images to Silicon Graphics RLE
  format by calculating the dynamic range and applying the auxiliary program
  fromge.c.
  Copyright (c) 1995 Matthew Belmonte
  User interface based on code by Brian Egaas and Mark Chaussee.
  Compile with "cc -O2 -o gecon gecon.c -lforms -lX11 -lm".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include <X11/X.h>
#include <gl/gl.h>
#include <gl/image.h>
#include <forms.h>
extern int errno;

static char fromge[] = "/usr/local/bin/fromge";

FL_FORM *translator;
FL_OBJECT *object,
	*indirectory,
	*outdirectory1,
	*outdirectory2,
	*browse_indir,
	*browse_outdir1,
	*browse_outdir2,
	*number,
	*width,
	*height,
	*stripbytes,
	*echo,
	*single_echo,
	*double_echo,
	*opt,
	*view_opt,
	*seg_opt,
	*start,
	*quit;

/* Form definition file generated with fdesign. */
void create_form_translator(void)
{
  FL_OBJECT *obj;
  translator = fl_bgn_form(FL_NO_BOX,800.0,620.0);
  obj = fl_add_box(FL_UP_BOX,0.0,0.0,800.0,620.0,"");
    fl_set_object_color(obj,FL_SLATEBLUE,FL_BLUE);
    fl_set_object_lsize(obj,11.000000);
  obj = fl_add_box(FL_UP_BOX,40.0,220.0,720.0,180.0,"");
    fl_set_object_lsize(obj,11.000000);
  obj = fl_add_text(FL_NORMAL_TEXT,180.0,540.0,430.0,80.0,"GE Image File Translator");
    fl_set_object_boxtype(obj,FL_UP_BOX);
    fl_set_object_lcol(obj,0);
    fl_set_object_lsize(obj,24.000000);
    fl_set_object_align(obj,FL_ALIGN_CENTER);
  indirectory = obj = fl_add_input(FL_NORMAL_INPUT,180.0,505.0,410.0,30.0,"Input");
    fl_set_object_lsize(obj,11.000000);
  browse_indir = obj = fl_add_lightbutton(FL_NORMAL_BUTTON,10.0,505.0,90.0,30.0,"Browse");
    fl_set_object_lsize(obj,11.000000);
  outdirectory1 = obj = fl_add_input(FL_NORMAL_INPUT,180.0,470.0,410.0,30.0,"Output");
    fl_set_object_lsize(obj,11.000000);
  browse_outdir1 = obj = fl_add_lightbutton(FL_NORMAL_BUTTON,10.0,470.0,90.0,30.0,"Browse");
    fl_set_object_lsize(obj,11.000000);
  outdirectory2 = obj = fl_add_input(FL_NORMAL_INPUT,180.0,435.0,410.0,30.0,"2nd Output");
    fl_set_object_lsize(obj,11.000000);
  browse_outdir2 = obj = fl_add_lightbutton(FL_NORMAL_BUTTON,10.0,435.0,90.0,30.0,"Browse");
    fl_set_object_lsize(obj,11.000000);
  number = obj = fl_add_input(FL_NORMAL_INPUT,710.0,470.0,60.0,50.0,"Number of\n Images");
    fl_set_object_lsize(obj,18.000000);
  obj = fl_add_box(FL_NO_BOX,230.0,340.0,310.0,40.0,"Translation Parameters");
    fl_set_object_lsize(obj,18.000000);
  width = obj = fl_add_input(FL_NORMAL_INPUT,110.0,300.0,60.0,30.0,"Width");
    fl_set_object_lsize(obj,11.000000);
  height = obj = fl_add_input(FL_NORMAL_INPUT,110.0,260.0,60.0,30.0,"Height");
    fl_set_object_lsize(obj,11.000000);
  stripbytes = obj = fl_add_input(FL_NORMAL_INPUT,520.0,300.0,70.0,30.0,"File Header Bytes");
    fl_set_object_lsize(obj,11.000000);
  echo = fl_bgn_group();
  single_echo = obj = fl_add_lightbutton(FL_RADIO_BUTTON,620.0,350.0,110.0,30.0,"Single Echo");
    fl_set_object_lsize(obj,11.000000);
  double_echo = obj = fl_add_lightbutton(FL_RADIO_BUTTON,620.0,320.0,110.0,30.0,"Double Echo");
    fl_set_object_lsize(obj,11.000000);
  fl_end_group();

  opt = fl_bgn_group();
  view_opt = obj = fl_add_lightbutton(FL_RADIO_BUTTON,620.0,270.0,110.0,30.0,"Looks Pretty");
  fl_set_object_lsize(obj,11.000000);
  seg_opt = obj = fl_add_lightbutton(FL_RADIO_BUTTON,620.0,240.0,110.0,30.0,"Segments Well");
  fl_set_object_lsize(obj,11.000000);
  fl_end_group();

  start = obj = fl_add_button(FL_NORMAL_BUTTON,241.0,120.0,290.0,40.0,"Do It");
    fl_set_object_lsize(obj,16.000000);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  quit = obj = fl_add_button(FL_NORMAL_BUTTON,340.0,30.0,100.0,30.0,"Quit");
    fl_set_object_lsize(obj,16.000000);
  fl_end_form();
}

/*Update the given histogram by considering all 'numpixels' pixels in
  the 16-bit binary image file 'imgname', whose header occupies 'offset'
  bytes.*/
static void update_histogram(imgname, numpixels, offset, histogram)
char *imgname;
int numpixels;
long offset;
unsigned long *histogram;
  {
  register int pixel;
  register FILE *imgfile;
  unsigned short pixel_value;
  imgfile = fopen(imgname, "r");
  fseek(imgfile, offset, SEEK_SET);
  for(pixel = 0; pixel != numpixels; pixel++)
    {
    if(fread(&pixel_value, sizeof(short), 1, imgfile) != 1)
      {
      fprintf(stderr, "Error: unexpected EOF in image file %s\n", imgname);
      fclose(imgfile);
      exit(errno);
      }
    histogram[pixel_value]++;
    }
  fclose(imgfile);
  }

/*compute the number of modes of the given histogram*/
static long modality(histogram)
unsigned long *histogram;
  {
  register long i, n;
  int sign, next_sign;
  n = 0L;
  sign = 0;
  for(i = 1; i != 0x10000; i++)
    {
    next_sign = histogram[i]-histogram[i-1];
    if(next_sign != 0)
      {
      if((sign > 0) && (next_sign < 0))
	n++;
      sign = next_sign;
      }
    }
  return(n);
  }

/*do a three-point smooth on the given histogram*/
static void smooth_histogram(histogram)
unsigned long *histogram;
  {
  register long i;
  unsigned long buf[2];
  buf[0] = (histogram[0]+histogram[1])/2L;
  for(i = 1; i != 0xffff; i++)
    {
    buf[1] = (histogram[i-1]+histogram[i]+histogram[i+1])/3L;
    histogram[i-1] = buf[0];
    buf[0] = buf[1];
    }
  histogram[0xffff] = (histogram[0xfffe]+histogram[0xffff])/2L;
  histogram[0xfffe] = buf[0];
  }

/*smooth the given histogram iteratively until it has 'num_modes' or fewer
  modes*/
static void smooth_to_n_modes(histogram, num_modes)
unsigned long *histogram;
int num_modes;
  {
  while(modality(histogram) > num_modes)
    smooth_histogram(histogram);
  }

/*Find the optimal values of the static offset and the dynamic range for
  the projection from 16-bit data to 8-bit data.*/
void find_optimal_range(indir, nimgs, numpixels, offset, mode, min, max, min2, max2, segmenting)
char *indir;
int nimgs, numpixels;
long offset;
int mode;
char *min, *max, *min2, *max2;
int segmenting;
  {
  register long i, sum;
  register char *filename;
  long acc;
  char pathname[MAXPATHLEN];
  unsigned long histogram[0x10000], histogram2[0x10000];
/*initialise histograms*/
  for(i = 0; i != 0x10000; i++)
    histogram[i] = 0;
  if(mode && !segmenting)
    for(i = 0; i != 0x10000; i++)
      histogram2[i] = 0;
/*set up path name*/
  filename = pathname;
  i = 0;
  while(indir[i] != '\0')
    *(filename++) = indir[i++];
  *(filename++) = '/';
  *(filename++) = 'I';
  *(filename++) = '.';
/*scan images*/
  if(segmenting)
    {
  /*use the middle image only - can't use all of them, because of axial
    decrease in intensity*/
    sprintf(filename, "%03d", (1+nimgs)/2);
    update_histogram(pathname, numpixels, offset, histogram);
    smooth_to_n_modes(histogram, 5);
  /*set the minimum intensity all the way down to zero*/
    min[0] = '0'; min[1] = '\0';
  /*find the greatest modal intensity - this is the air peak*/
    sum = 0L;
    for(i = 0; i != 0xffff; i++)
      if(histogram[i] >= sum)
	{
	sum = histogram[i];
	acc = 1+i;
	}
  /*continue to the base of the air peak*/
    while((acc != 0xffff) && (histogram[acc] >= histogram[1+acc]))
      acc++;
  /*find the second greatest modal intensity - this is the grey matter peak*/
    sum = 0L;
    for(i = acc; i != 0x10000; i++)
      if(histogram[i] >= sum)
	{
	sum = histogram[i];
	acc = i;
	}
  /*put the grey matter peak in the middle of the converted range, and round
    the resulting maximum intensity to a multiple of 256*/
    acc = (acc*2L+0x80L)&~0xffL;
    sprintf(max, "%ld", acc);
  /*use the PD-derived range for the T2 series also*/
    if(mode)
      {
      strcpy(min2, min);
      strcpy(max2, max);
      }
    }
  else
  /*not segmenting*/
    {
    for(i = 1; i <= nimgs; i++)
      {
      sprintf(filename, "%03ld", i);
      if(mode && (i&1 == 0))
	update_histogram(pathname, numpixels, offset, histogram2);
      else
	update_histogram(pathname, numpixels, offset, histogram);
      }
  /*Exclude the 0.025% of the pixel intensities in each tail, in order to
    maximise resolution of the central intensity values.*/
    for(sum=0, i=0; sum <= nimgs*numpixels/4000; sum += histogram[i++])
      ;
    acc = i-1;
    sprintf(min, "%ld", acc);
    for(sum=0, i=0x10000; sum <= nimgs*numpixels/4000; sum += histogram[--i])
      ;
    sprintf(max, "%ld", i);
    if(mode)
      {
      for(sum=0, i=0; sum <= nimgs*numpixels/4000; sum += histogram2[i++])
	;
      acc = i-1;
      sprintf(min2, "%ld", acc);
      for(sum=0, i=0x10000; sum <= nimgs*numpixels/4000; sum += histogram2[--i])
	;
      sprintf(max2, "%ld", i);
      }
    }
  }

int get_number_of_source_files(dirp)
const char *dirp;
  {
  register int nimgs, i;
  char dirname[MAXPATHLEN];
  int dirnamelen;
/*get the new directory name*/
  for(dirnamelen = 0; dirp[dirnamelen] != '\0'; dirnamelen++)
    dirname[dirnamelen] = dirp[dirnamelen];
/*the first image in the series, I.001, must be present*/
  strcpy(dirname+dirnamelen, "/I.001");
  nimgs = 1;
  i = open(dirname, O_RDONLY);
/*count the number of images*/
  while(i != -1)
    {
    close(i);
    sprintf(dirname+dirnamelen+3, "%03d", ++nimgs);
    i = open(dirname, O_RDONLY);
    }
  return(--nimgs);
  }

/*Get a pathname from a browser, and then slice off the file name, leaving the
  directory name by itself.*/
const char *get_dirname_from_browser(initial_dirname)
const char *initial_dirname;
  {
  const char *pathname;
  static char buf[MAXPATHLEN];
  register int i, rightmost_separator;
  pathname = fl_show_fselector("select any file inside the desired directory:", initial_dirname, "*", "");
  rightmost_separator = -1;
  if(pathname != (const char *)0)
    for(i = 0; pathname[i] != '\0'; i++)
      {
      buf[i] = pathname[i];
      if(pathname[i] == '/')
	rightmost_separator = i;
      }
  if(rightmost_separator == -1)
    return(initial_dirname);
  buf[rightmost_separator? rightmost_separator: 1] = '\0';
  return(buf);
  }

void main(argc, argv)
int argc;
char **argv;
  {
  register int i;
  int	nimgs,	/*# of images -- this is the # of echoes times the # of slices*/
	mode,	/*0 for single echo, 1 for double echo*/
	segmenting; /*0 to optimise range for viewing, 1 to optimise smoothness for segmenting*/
const char *xsz, /*image width (ASCII)*/
	*ysz,	/*image height (ASCII)*/
	*skip;	/*# of bytes in header (ASCII)*/
  char	nimgs_string[10],
	min[6],	/*minimum valid pixel value (ASCII)*/
	second_echo_min[6], /*minimum valid pixel value for 2nd echo*/
	max[6],	/*maximum valid pixel value (ASCII)*/
	second_echo_max[6], /*minimum valid pixel value for 2nd echo*/
	indir[MAXPATHLEN],
	outdir1[MAXPATHLEN],
	outdir2[MAXPATHLEN];
  int indirlen, outdir1len, outdir2len;
/*initialise the input form*/
  fl_initialize(&argc, argv, "gecon", 0, 0); /*OLD VERSION WAS fl_initialize(argv[0], "gecon", 0, 0, &argc, argv);*/
  create_form_translator();
  fl_show_form(translator, FL_PLACE_SIZE,TRUE, "Translator");
  fl_hide_object(outdirectory2);
  fl_hide_object(browse_outdir2);
  getwd(indir);
  fl_set_input(indirectory, indir);
  fl_set_input(outdirectory1, strcat(strcpy(outdir1, indir), "/data"));
  fl_set_input(outdirectory2, strcpy(outdir2, outdir1));
  nimgs = get_number_of_source_files(indir);
  sprintf(nimgs_string, "%d", nimgs);
  fl_set_input(number, nimgs_string);
  fl_set_input(width, "256");
  fl_set_input(height, "256");
  fl_set_input(stripbytes, "14336");
  fl_set_button(single_echo, 1);
  mode = 0;
  fl_set_button(view_opt, 1);
  segmenting = 0;
/*accept input*/
  while((object = fl_do_forms()) != quit)
    {
  /*process a change of the input directory*/
    if(object == indirectory)
      {
      i = get_number_of_source_files(fl_get_input(indirectory));
    /*if the directory is readable and contains at least one readable
      image, accept the new directory name and set the number of
      images accordingly*/
      if(i != 0)
	{
	nimgs = i;
	strcpy(indir, fl_get_input(indirectory));
	sprintf(nimgs_string, "%d", nimgs);
	fl_set_input(number, nimgs_string);
	}
      else
      /*if no images could be read, disallow the change of directory*/
	fl_set_input(indirectory, indir);
      }
    else if(object == browse_indir)
      {
      const char *dirp;
      dirp = get_dirname_from_browser(indir);
      i = get_number_of_source_files(dirp);
      if(i != 0)
	{
	nimgs = i;
	strcpy(indir, fl_get_input(indirectory));
	sprintf(nimgs_string, "%d", nimgs);
	fl_set_input(number, nimgs_string);
	strcpy(indir, dirp);
	fl_set_input(indirectory, indir);
	}
      }
    else if(object == browse_outdir1)
      fl_set_input(outdirectory1, get_dirname_from_browser(fl_get_input(outdirectory1)));
    else if(object == browse_outdir2)
      fl_set_input(outdirectory2, get_dirname_from_browser(fl_get_input(outdirectory2)));
    else if(object == double_echo)
      {
      fl_show_object(outdirectory2);
      fl_show_object(browse_outdir2);
      mode = 1;
      }
    else if(object == single_echo)
      {
      fl_hide_object(outdirectory2);
      fl_hide_object(browse_outdir2);
      mode = 0;
      }
    else if(object == seg_opt)
      segmenting = 1;
    else if(object == view_opt)
      segmenting = 0;
    else if(object == start)
      {
      nimgs = atoi(fl_get_input(number));
      strcpy(outdir1, fl_get_input(outdirectory1));
      strcpy(outdir2, fl_get_input(outdirectory2));
      xsz = fl_get_input(width);
      ysz = fl_get_input(height);
      skip = fl_get_input(stripbytes);
      find_optimal_range(indir, nimgs, atoi(xsz)*atoi(ysz), atol(skip), mode, min, max, second_echo_min, second_echo_max, segmenting);
    /*create output directories, if they don't already exist*/
      mkdir(outdir1, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH);
      if(mode == 1)
	mkdir(outdir2, S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH);
    /*set up path names into those directories*/
      indirlen = strlen(indir);
      indir[indirlen] = '/';
      indir[indirlen+1] = 'I';
      indir[indirlen+2] = '.';
      outdir1len = strlen(outdir1);
      outdir1[outdir1len] = '/';
      if(mode)
	{
	outdir2len = strlen(outdir2);
	outdir2[outdir2len] = '/';
	}
    /*convert images*/
      for(i = 1; i <= nimgs; i++)
	{
	sprintf(indir+indirlen+3, "%03d", i);
	sprintf((mode && (i%2 == 0))? outdir2+outdir2len+1: (outdir1+outdir1len+1), "%d.bw", (mode? (i+1)/2: i));
	printf("%s: ", indir+indirlen+3);
	fflush(stdout);
	if(fork() == 0)
	  execl(fromge, "fromge", indir, ((mode&&(i%2==0))? outdir2: outdir1), xsz, ysz, skip, ((mode&&(i%2==0))? second_echo_min: min), ((mode&&(i%2==0))? second_echo_max: max), (char *)0);
	wait((int *)0);
	}
      indir[indirlen] = '\0';
      outdir1[outdir1len] = '\0';
      if(mode)
	outdir2[outdir2len] = '\0';
      printf("Converted %d %sx%s images with header length %s\nfrom %s\nto %s%s%s.\nValid pixel range was %s..%s\n", nimgs, xsz, ysz, skip, indir, outdir1, (mode? " and ": ""), (mode? outdir2: ""), min, max);
      if(mode)
	printf("for first echo, and %s..%s for second echo\n", second_echo_min, second_echo_max);
      }
    }
  exit(0);
  }
