/*segment.c - front-end for Brian Egaas's segmentation suite.
  Copyright (c) 1995 Matthew Belmonte

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

/*for best performance, change the 'b' entry in /usr/lib/cron/queuedefs to
b.1j20n
*/

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/wait.h>

static char model[] = "/usr/local/bin/model2",
	    siscale[] = "/usr/local/bin/siscale",
	    correct[] = "/usr/local/bin/correct",
	    seg[] = "/usr/local/bin/3dseg4",
	    date[] = "/bin/date",
	    rm[] = "/bin/rm",
	    editor[] = "/usr/sbin/jot",
	    viewer[] = "/usr/people/4Dgifts/iristools/imgtools/scope";

#define NUM_FILES 9 /*if you change this, also change the execl call to rm*/
#define OUT_FILE (files[7])
#define ERR_FILE (files[8])
static char files[NUM_FILES][15] = {"2dhistck.bw", "parameters.dat", "scale.dat", "pdc", "t2c", "seg", "seg2", "segment.out", "segment.err"};

void batch()
  {
  if(fork() != 0)
    exit(0);
  nice(40);
  freopen(OUT_FILE, "w", stdout);
  freopen(ERR_FILE, "w", stderr);
  }

void main(argc, argv)
int argc;
char **argv;
  {
  int image, status, make_clean, interact;
  FILE *batchq;
  char num_images[5], pdfile[11], t2file[11];
  struct stat stbuf;

/*process options*/
  if(getuid() == 0)
    {
    fprintf(stderr, "%s cannot be run as root.\n", *argv);
    exit(1);
    }
  make_clean = 0;
  interact = 0;
  while((argc > 1) && (argv[1][0] == '-'))
    {
    register int i;
    for(i = 1; argv[1][i] != '\0'; i++)
      switch(argv[1][i])
	{
	case 'f': make_clean = 1; break;
	case 'i': interact = 1; break;
	default: fprintf(stderr, "unrecognised option '%c'\n", argv[1][i]);
	}
    argc--;
    argv++;
    }

/*change directory, if it was given as an argument*/
  if((argc > 1) && (chdir(argv[1]) == -1))
    {
    perror(argv[1]);
    exit(errno);
    }

/*remove old data*/
  if(make_clean)
    {
    if(fork() == 0)
      execl(rm, "rm", "-rf", files[0], files[1], files[2], files[3], files[4], files[5], files[6], files[7], files[8], (char *)0);
    }
  else
    {
    for(argc = 0; argc != NUM_FILES; argc++)
      if(stat(files[argc], &stbuf) == 0)
	{
	fprintf(stderr, "Use the -f option if you want to overwrite old data.\n");
	exit(1);
	}
    }

/*determine number of images*/
  *pdfile = 'p';
  pdfile[1] = 'd';
  *t2file = 't';
  t2file[1] = '2';
  pdfile[2] = t2file[2] = '/';
  image = 0;
  do
    {
    sprintf(pdfile+3, "%03d.bw", ++image);
    sprintf(t2file+3, "%03d.bw", image);
    }
  while((stat(pdfile, &stbuf) == 0) && (stat(t2file, &stbuf) == 0));
  image--;
  do
    {
    sprintf(pdfile+3, "%02d.bw", ++image);
    sprintf(t2file+3, "%02d.bw", image);
    }
  while((stat(pdfile, &stbuf) == 0) && (stat(t2file, &stbuf) == 0));
  image--;
  do
    {
    sprintf(pdfile+3, "%d.bw", ++image);
    sprintf(t2file+3, "%d.bw", image);
    }
  while((stat(pdfile, &stbuf) == 0) && (stat(t2file, &stbuf) == 0));
  if(--image == 0)
    {
    perror((char *)0);
    exit(errno);
    }
  sprintf(num_images, "%d", image);

/*wait for rm to finish, then run in the background if -i wasn't specified*/
  if(make_clean)
    wait(&status);
  if(!interact)
    batch();

/*model2*/
  if(fork() == 0)
    {
    execl(model, "model", "t2", "pd", num_images, (char *)0);
    exit(errno);
    }
  do
    wait(&status);
  while(WIFSTOPPED(status));
  if(status != 0)
    exit(status);

/*siscale pd*/
  if(fork() == 0)
    {
    execl(siscale, "siscale", "pd", num_images, (char *)0);
    exit(errno);
    }
  do
    wait(&status);
  while(WIFSTOPPED(status));
  if(status != 0)
    exit(status);

/*correct*/
  if(fork() == 0)
    {
    execl(correct, "correct", num_images, (char *)0);
    exit(errno);
    }
  do
    wait(&status);
  while(WIFSTOPPED(status));
  if(status != 0)
    exit(status);

/*date*/
  if(fork() == 0)
    {
    execl(date, "date", (char *)0);
    exit(errno);
    }
  wait(&status);

/*check parameters*/
  if(interact)
    {
    if(fork() == 0)
      execl(editor, editor, "parameters.dat", (char *)0);
    if(fork() == 0)
      execl(viewer, viewer, "2dhistck.bw", (char *)0);
    wait((int *)0);
    wait((int *)0);
    printf("Proceed with segmentation using these parameters [n]? ");
    if((getchar() | 0x20) != 'y')
      exit(0);
    batch();
    }

/*batch 3dseg4*/
  fflush(stdout);
  fflush(stderr);
  batchq = popen("/bin/batch", "w");
  fprintf(batchq, "date >>%s\nif %s pdc t2c %s >>%s 2>>%s\nthen\necho segmentation completed with no errors\nelse\necho segmentation aborted with error $?\nfi\npwd\ndate >>%s", OUT_FILE, seg, num_images, OUT_FILE, ERR_FILE, OUT_FILE);
  pclose(batchq);
  exit(0);
  }
