/*clump.c - remove all high-intensity pixels except those that occur in clumps.
  Copyright (C) 1996, 2002 Matthew Belmonte

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

#include <stdlib.h>
#include <errno.h>
#include "sweet.h"

int check_corner(imgdata, y, x, dy, dx, len, threshold)
short **imgdata;
int y, x, dy, dx, len;
  {
  register int h, i, j;
  h = y;
  j = x;
  for(i = 0; i != len; i++)
    {
    if((imgdata[y][j] <= threshold) || (imgdata[h][x] <= threshold))
      return(1);
    h += dy;
    j += dx;
    }
  return(0);
  }

int main(argc, argv)
int argc;
char **argv;
  {
  register int j, h, col, row;
  short threshold;
  int diameter, num_rows, num_cols;
  IMAGE *img;
  short **inimg_data, *outimg_data;
  if(argc < 4)
    {
    fprintf(stderr, "usage: %s <inimg.bw> <outimg.bw> <diameter> [threshold]\nwhere <diameter> is the minimum diameter (in pixels) of a true clump,\nand [threshold] is the maximum intensity of non-clump pixels\n", *argv);
    exit(1);
    }
  diameter = atoi(argv[3]);
  threshold = ((argc > 4)? (short)atoi(argv[4]): 0);
  if((img = iopen(argv[1], "r")) == (IMAGE *)0)
    {
    perror(argv[1]);
    exit(errno);
    }
  num_cols = img->xsize;
  num_rows = img->ysize;
  inimg_data = (short **)calloc(num_rows, sizeof(short *));
  for(row = 0; row != num_rows; row++)
    {
    inimg_data[row] = (short *)calloc(num_cols, sizeof(short));
    getrow(img, inimg_data[row], row, 0);
    }
  iclose(img);
  if((img = iopen(argv[2], "w", RLE(1), 3, num_cols, num_rows, 1)) == (IMAGE *)0)
    {
    perror(argv[2]);
    exit(errno);
    }
  outimg_data = (short *)calloc(num_cols, sizeof(short));
  for(row = 0; row != diameter/2; row++)
    {
    for(col = 0; col != num_cols; col++)
      outimg_data[col] = ((inimg_data[row][col] > threshold)? threshold: inimg_data[row][col]);
    putrow(img, outimg_data, row, 0);
    }
  while(row != num_rows-diameter/2)
    {
    for(col = 0; col != diameter/2; col++)
      outimg_data[col] = ((inimg_data[row][col] > threshold)? threshold: inimg_data[row][col]);
    while(col != num_cols-diameter/2)
      {
      outimg_data[col] = inimg_data[row][col];
      if(inimg_data[row][col] > threshold)
	{
	for(h = (1-diameter)/2; h <= (diameter-1)/2; h++)
	  for(j = (1-diameter)/2; j <= (diameter-1)/2; j++)
	    if(inimg_data[row+h][col+j] <= threshold)
	      outimg_data[col] = threshold;
	if(!(diameter & 1)
	 && (outimg_data[col] > threshold)
	 && check_corner(inimg_data, row-diameter/2, col-diameter/2, 1, 1, diameter, threshold)
	 && check_corner(inimg_data, row-diameter/2, col+diameter/2, 1, -1, diameter, threshold)
	 && check_corner(inimg_data, row+diameter/2, col-diameter/2, -1, 1, diameter, threshold)
	 && check_corner(inimg_data, row+diameter/2, col+diameter/2, -1, -1, diameter, threshold))
	  outimg_data[col] = threshold;
	}
      col++;
      }
    while(col != num_cols)
      {
      outimg_data[col] = ((inimg_data[row][col] > threshold)? threshold: inimg_data[row][col]);
      col++;
      }
    putrow(img, outimg_data, row++, 0);
    }
  while(row != num_rows)
    {
    for(col = 0; col != num_cols; col++)
      outimg_data[col] = ((inimg_data[row][col] > threshold)? threshold: inimg_data[row][col]);
    putrow(img, outimg_data, row++, 0);
    }
  iclose(img);
  free(outimg_data);
  for(row = 0; row != num_rows; row++)
    free(inimg_data[row]);
  free(inimg_data);
  return 0;
  }
