/*colourise - a hack to add false colour to a greyscale image, and, optionally,
  to apply a threshold, all pixels less than or equal to which become black.
  Copyright (C) 1996, 2002 Matthew Belmonte

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "sweet.h"

#define XMAX 1024

int main(argc, argv)
int argc;
char **argv;
  {
  register int row, col;
  short	threshold,	/*points <= threshold are black*/
	saturation,	/*points >= saturation are pure red*/
	midrange;
  IMAGE *bw, *rgb;
  short bwbuf[XMAX], cbuf[XMAX];
  if(argc < 3)
    {
    fprintf(stderr, "usage: %s <image.bw> <image.rgb> [threshold] [saturation]\n", *argv);
    exit(1);
    }
  threshold = ((argc > 3)? atoi(argv[3]): -1);
  saturation = ((argc > 4)? atoi(argv[4]): 255);
  if(saturation-threshold < 4)
    {
    fprintf(stderr, "bad range %d..%d\n", threshold, saturation);
    exit(1);
    }
  midrange = threshold+(saturation-threshold)/2-1;
  if((bw = iopen(argv[1], "r")) == (IMAGE *)0)
    {
    perror(argv[1]);
    exit(errno);
    }
  if(bw->xsize > XMAX)
    {
    iclose(bw);
    fprintf(stderr, "recompile %s with XMAX=%d\n", *argv, bw->xsize);
    exit(1);
    }
  if((rgb = iopen(argv[2], "w", RLE(1), 3, bw->xsize, bw->ysize, 3)) == (IMAGE *)0)
    {
    perror(argv[2]);
    iclose(bw);
    exit(errno);
    }
  for(row = 0; row != bw->ysize; row++)
    {
    getrow(bw, bwbuf, row, 0);
  /*red*/
    for(col = 0; col != bw->xsize; col++)
      cbuf[col] = ((bwbuf[col] <= midrange)? 0: (bwbuf[col] >= saturation)? 0xff: 0xff*(bwbuf[col]-midrange)/(saturation-midrange));
    putrow(rgb, cbuf, row, 0);
  /*green*/
    for(col = 0; col != bw->xsize; col++)
      cbuf[col] = (((bwbuf[col] <= threshold)||(bwbuf[col] >= saturation))? 0: (bwbuf[col] <= midrange)? (0xff*(bwbuf[col]-threshold))/(midrange-threshold): (0xff*(saturation-bwbuf[col]))/(saturation-midrange));
    putrow(rgb, cbuf, row, 1);
  /*blue*/
    for(col = 0; col != bw->xsize; col++)
      cbuf[col] = ((bwbuf[col] <= threshold) || (bwbuf[col] > midrange))? 0: 0xff-(0xff*(bwbuf[col]-threshold))/(midrange-threshold);
    putrow(rgb, cbuf, row, 2);
    }
  iclose(rgb);
  iclose(bw);
  return 0;
  }
