/*fromge.c - convert a G.E. Signa MR image to Silicon Graphics RLE format.
  Copyright (C) 1995, 2002 Matthew Belmonte

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <netinet/in.h>
#include "sweet.h"

#define IMG_WIDTH 256
#define IMG_HEIGHT 256
#define IMG_OFFSET 14336
#define IMG_MINVAL 0
#define IMG_MAXVAL 1024
#define MAX_WIDTH 1024

int main(argc, argv)
int argc;
char **argv;
  {
  register int x, y;
  register long num_above_maxval, num_below_minval;
  int width, height;
  long offset;
  FILE *ge;
  IMAGE *rle;
  unsigned short minval, maxval, rowbuf[MAX_WIDTH];
  if(argc < 3)
    {
    fprintf(stderr, "usage: %s <inimg> <outimg> <width> <height> <offset> <min. pixel value> <max. pixel value>\ndefault parameter settings are as follows:\n\twidth %d\n\theight %d\n\toffset %d\n\tmin. %d\n\tmax. %d\n", *argv, IMG_WIDTH, IMG_HEIGHT, IMG_OFFSET, IMG_MINVAL, IMG_MAXVAL);
    exit(1);
    }
/*fill parameters with givens or with default values*/
  width = (argc >= 4)? atoi(argv[3]): IMG_WIDTH;
  if(width > MAX_WIDTH)
    {
    fprintf(stderr, "Recompile %s with MAX_WIDTH=%d\n", *argv, width);
    exit(1);
    }
  height = (argc >= 5)? atoi(argv[4]): IMG_HEIGHT;
  offset = (argc >= 6)? atol(argv[5]): IMG_OFFSET;
  minval = (argc >= 7)? (unsigned short)(atoi(argv[6])): IMG_MINVAL;
  maxval = (argc >= 8)? (unsigned short)(atoi(argv[7])): IMG_MAXVAL;
  if((width <= 0) || (height <= 0) || (offset < 0L) || (maxval < minval))
    {
    fprintf(stderr, "%s: screwy parameter values\n", *argv);
    exit(1);
    }
/*open the GE Signa image*/
  ge = fopen(argv[1], "r");
  if(ge == NULL)
    {
    perror(argv[1]);
    exit(errno);
    }
  fseek(ge, offset, SEEK_SET);
/*create a new SGI image*/
  rle = iopen(argv[2], "w", RLE(1), 3, width, height, 1);
  if(rle == NULL)
    {
    perror(argv[2]);
    fclose(ge);
    exit(errno);
    }
  num_below_minval = 0L;
  num_above_maxval = 0L;
  for(y = 0; y != height; y++)
    {
  /*read a row of pixels*/
    if(fread(rowbuf, sizeof(short), width, ge) != width)
      {
      iclose(rle);
      unlink(argv[2]);
      fclose(ge);
      fprintf(stderr, "size of GE file %s inconsistent with header length %d, width %d, and height %d\n", argv[1], offset, width, height);
      exit(1);
      }
  /*transform the row to the new, 8-bit range*/
    for(x = 0; x != width; x++)
      {
      rowbuf[x] = ntohs(rowbuf[x]); /*swap bytes if necessary*/
      if(rowbuf[x] < minval)
	{
	rowbuf[x] = minval;
	num_below_minval++;
	}
      else if(rowbuf[x] > maxval)
	{
	rowbuf[x] = maxval;
	num_above_maxval++;
	}
      rowbuf[x] = (unsigned short)(255L*(long)(rowbuf[x]-minval)/(long)(maxval-minval));
      }
  /*write the row*/
    putrow(rle, rowbuf, height-1-y, 0);
    }
/*close files*/
  iclose(rle);
  fclose(ge);
/*report tallies of pixels out of range*/
  printf("%ld pixels less than 0, %ld pixels greater than 255\n", num_below_minval, num_above_maxval);
  return 0;
  }
