/*sweet.c - GL API compatibility for Michael Sweet's SGI image format library.
  Copyright (C) 2002 Matthew Belmonte <belmonte@mit.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.*/

#include <stdarg.h>
#include "sgi.h"
#include "sweet.h"

IMAGE *iopen(const char *file, const char *mode, ...)
  {
  va_list ap;
  int imode,			/*SGI_READ or SGI_WRITE*/
	comp,			/*compression scheme*/
	bpp,			/*bits per pixel*/
	xsize, ysize, zsize;	/*dimensions*/
  if(*mode == 'w')		/*if writing, we need more parameters*/
    {
    imode = SGI_WRITE;
    va_start(ap, mode);
    bpp = va_arg(ap, unsigned);
    comp = (((bpp & 0xff00) == 0x0100)? SGI_COMP_RLE: SGI_COMP_NONE);
    bpp &= 0xff;
    va_arg(ap, unsigned);	/*dimension parameter is redundant & not used*/
    xsize = (int)(va_arg(ap, unsigned));
    ysize = (int)(va_arg(ap, unsigned));
    zsize = (int)(va_arg(ap, unsigned));
    va_end(ap);
    }
  else				/*if reading, we need only two parameters*/
    {
    imode = SGI_READ;
    bpp = comp = xsize = ysize = zsize = 0;
    }
  return(sgiOpen(file, imode, comp, bpp, xsize, ysize, zsize));
  }
